#Beginnings of a proxy server

#Notes on things done:
	#See previous version in VSS for these comments.

import sys
import traceback
import urllib
import string
import re
import os
import tempfile
import socket
import time

import processHtml

PORT = 8578			#arbitrary, high number

def writeDebugInfo(msg):
	debugFile = open("c:/debug.txt", "a+")
	debugFile.write(msg)
	debugFile.close()
	return

def tracebackAsString(limit):
	type, value, tb = sys.exc_info()
	aList = traceback.format_exception(type, value, tb, limit)
	formattedString = ""
	for s in aList:
		formattedString = formattedString + s + "\n"
	return formattedString

def getQueryStringForUrlOpen(queryString):
	#remove OUR query string parameters but keep any we should use
	#when opening the url
	items = string.split(queryString, "&")
	queryStringForUrlOpen = ""
	for item in items:
		if item[:9] == "k0olbase=":
			pass
		elif item[:8] == "k0olurl=":
			pass
		else:
			if queryStringForUrlOpen:
				queryStringForUrlOpen = queryStringForUrlOpen + "&"
			queryStringForUrlOpen = queryStringForUrlOpen + item
	if queryStringForUrlOpen:
		queryStringForUrlOpen = "?" + queryStringForUrlOpen
	return queryStringForUrlOpen

def saveAndIndexSurfedSite(storageFolder, tempDirectory, siteHtmlString, siteUrl,
							authenticatedUser, serverName):
	#Saves page in temp file and "calls" background process to do compressing, saving, indexing.
	authenticatedUserName = `authenticatedUser`

	#add some meta info. to the file before storing it
	headEndTagPat = re.compile(r'</head>', re.IGNORECASE | re.DOTALL)
	match = headEndTagPat.search(siteHtmlString)
	if match:
		startHeadTagEnd = match.start()
		s = siteHtmlString[:startHeadTagEnd]
		s = s + '<meta name="origK0olUrl" content="' + siteUrl + '">\n'
		s = s + siteHtmlString[startHeadTagEnd:]

	#create a temp file where we'll store the info. for the storagerIndex to pick it up
	try:
		tempfile.tempdir = tempDirectory
		fn = tempfile.mktemp()
		#writeDebugInfo("temp file is " + `fn` + "\n")
		#PROBLEM:
		#try line buffering here--occasionally getting an IO error writing these files--not sure why
		f = open(fn, "w")
	except:
		writeDebugInfo("Error creating temporary file for surfed site storage.\n")
		return

	try:
		f.write(s)
		f.close()
		#Note: the storager program will delete this temp file when it is no longer needed.
	except:
		try:
			os.remove(fn)
		except:
			pass
		writeDebugInfo("Error writing surfed site data to temporary file.\n")
		return

	#talk to storagerIndexer process which will store the html in a ZODB for the authenticated
	#user and will index it
	try:
		clientsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		dateAndTime = time.ctime(time.time())
		storageRequest = ("store" + "|"
							+ authenticatedUserName + "|"
							+ fn + "|"
							+ storageFolder + "|"
							+ siteUrl + "|"
							+ dateAndTime)
		clientsock.connect(serverName, PORT)
		clientsock.send(storageRequest)
		clientsock.close()
	except Exception, data:
		writeDebugInfo("Error communicating with storager: " + `data` + "\n")
		try:
			clientsock.close()
		except:
			pass

	return

def proxy(self, REQUEST, RESPONSE, AUTHENTICATED_USER):
	#Zope external method
	#things to do:
		#see notes on previous version in VSS.

	queryString = REQUEST['QUERY_STRING']
	serverName = REQUEST["SERVER_NAME"]

	writeDebugInfo("-------------------------------------------------------------------------------------\n")

	#queryString sometimes is blank
	if queryString:

		#fix problem with queryString being quoted sometimes which throws off everything
		queryString = urllib.unquote(queryString)

		#fix problem with CGI scripts adding their query string and additional "?'s" to our query
		#string which has base and url info. in it
		queryString = string.replace(queryString, "?", "&")

		writeDebugInfo("Query string passed is " + queryString + "\n\n")

		#find the base address
		try:
			basePat = re.compile(r'k0olbase=(?P<base>[^&]*)', re.IGNORECASE | re.DOTALL)
			match = basePat.search(queryString)
			processHtml.origBaseUrl = match.group("base")
			writeDebugInfo("Current origBaseUrl as passed is " + processHtml.origBaseUrl + "\n\n")
		except AttributeError, data:
			#for testing/debugging
			return "Can't find k0olbase in queryString: " + "\n\n" + `REQUEST`

		#find the url, which may be absolute or relative
		try:
			urlPat = re.compile(r'k0olurl=(?P<url>[^&]*)', re.IGNORECASE | re.DOTALL)
			match = urlPat.search(queryString)
			url = match.group("url")
			writeDebugInfo("Current url as passed is " + url + "\n\n")
		except AttributeError, data:
			#for testing/debugging
			return "Can't find k0olurl in queryString: " + "\n\n" + `REQUEST`

	#queryString was blank; so get info. directly from REQUEST
	else:
		writeDebugInfo("No query string. Gettings args from REQUEST directly\n\n")
		processHtml.origBaseUrl = REQUEST['k0olbase']
		writeDebugInfo("Current origBaseUrl as passed is " + processHtml.origBaseUrl + "\n\n")
		url = REQUEST['k0olurl']
		writeDebugInfo("Current url as passed is " + url + "\n\n")

	#get query string minus base and url, which we already have; so we can pass it
	#on when we open the url
	queryStringForUrlOpen = getQueryStringForUrlOpen(queryString)
	writeDebugInfo("Query string for urlopen is " + queryStringForUrlOpen + "\n\n")

	#determine full url to open
	fullUrl = processHtml.constructAbsoluteUrl(url)

	#get contents of url
	try:
		#-----determine the new base URL from the current full URL-----

		processHtml.origBaseUrl = processHtml.getBaseUrlFromFullUrl(fullUrl)

		#-----open the URL-----

		tmp = fullUrl + queryStringForUrlOpen

		#note exactly what we're reading this time
		writeDebugInfo("Reading and converting " + tmp + "\n\n")

		urlFileObj = urllib.urlopen(tmp)

	except IOError, data:
		writeDebugInfo("\nIOError exception opening URL " + tmp + "\n\n")
		return "IOError exception opening URL " + tmp
	except Exception, data:
		return "Exception opening URL " + tmp

	list = urlFileObj.readlines()

	#determine if we were redirected; if so, the base url needs to be changed before
	#making the regex mods below
	urlAfterOpen = urlFileObj.geturl()
	writeDebugInfo("urlAfterOpen is " + urlAfterOpen + "\n\n")
	processHtml.origBaseUrl = processHtml.getBaseUrlFromFullUrl(urlAfterOpen)
	writeDebugInfo("final origBaseUrl before mods to html is " + processHtml.origBaseUrl + "\n\n")

	urlFileObj.close()

	writeDebugInfo("-------------------------------------------------------------------------------------\n")

	#make one big string to modify using regular expressions
	s = ""
	for item in list:
		s = s + item

	#save the surfed site in a user's personal ZODB before mods below which increase its size greatly;
	#911 !!!---speed improvement--this is assuming you have write-back cache on your hard drive; otherwise,
	#this will slow down the user seeing his page until the file is stored and indexed.
	storageFolder = self.aq_parent["storageFolder"]
	tempDirectory = self.aq_parent["tempDirectory"]
	saveAndIndexSurfedSite(storageFolder, tempDirectory, s, processHtml.origBaseUrl,
							AUTHENTICATED_USER, serverName)

	s = processHtml.proxifyHtml(s)

	return s